@@ -1,11 +1,29 @@
 #!/usr/bin/env ruby
-=begin
-   Copyright (C) 2001, 2002 Matt Armstrong.  All rights reserved.
-
-   Permission is granted for use, copying, modification, distribution,
-   and distribution of modified versions of this work as long as the
-   above copyright notice is included.
-=end
+#--
+#   Copyright (C) 2001, 2002, 2003 Matt Armstrong.  All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. The name of the author may not be used to endorse or promote products
+#    derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
+# NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
 
 require 'tests/testbase'
 require 'rmail/address'
@@ -80,6 +98,7 @@ class TestRMailAddress < TestBase
       $DEBUG = prev_debug
     end
     assert_kind_of(Array, results)
+    assert_kind_of(RMail::Address::List, results)
     results.each { |address|
       validate_interface(address)
     }
@@ -299,6 +318,16 @@ class TestRMailAddress < TestBase
 	  :local => 'bjbear71',
 	  :format => '"B.J.Major" <bjbear71@example.net>' } ] ]
 
+    validate_case\
+    ['jdoe@test   . example',
+      [ { :name => nil,
+	  :display_name => nil,
+	  :address => 'jdoe@test.example',
+	  :comments => nil,
+	  :domain => 'test.example',
+	  :local => 'jdoe',
+	  :format => 'jdoe@test.example' } ] ]
+
     validate_case\
     ['Mary Smith <@machine.tld:mary@example.net>, , jdoe@test   . example',
       [ { :name => 'Mary Smith',
@@ -765,6 +794,20 @@ class TestRMailAddress < TestBase
 	} ] ]
   end
 
+  def test_invalid_addresses()
+    # The display name isn't encoded -- bad, but we parse it.
+    validate_case\
+    ["\322\315\322 \312\353\363\341 <bar@foo.invalid>",
+      [ { :name => "\322\315\322 \312\353\363\341",
+	  :display_name => "\322\315\322 \312\353\363\341",
+	  :address => 'bar@foo.invalid',
+	  :comments => nil,
+	  :domain => 'foo.invalid',
+	  :local => 'bar',
+	  :format => "\"\322\315\322 \312\353\363\341\" <bar@foo.invalid>",
+	} ] ]
+  end
+
   def test_domain_literal
 
     validate_case\
@@ -1094,4 +1137,56 @@ class TestRMailAddress < TestBase
     assert_equal([ 'foo' ], a.comments)
   end
 
+  def test_rmail_address_compare
+    a1 = RMail::Address.new
+    a2 = RMail::Address.new
+
+    # Make sure it works on uninitialized addresses
+    assert_equal(0, a1 <=> a2)
+    assert_equal(a1, a2)
+
+    # Display name and comments make no difference
+    a1.display_name = 'foo'
+    a1.comments = 'bar'
+    assert_equal(0, a1 <=> a2)
+    assert_equal(a1, a2)
+
+    # a1 < a2
+    a1 = RMail::Address.new("Zeus <bob@example.com>")
+    a2 = RMail::Address.new("sally@example.com")
+    assert(a1 < a2)
+    assert(a1 != a2)
+
+    # Type coercion
+    assert(a1 < "zorton@example.com")
+    assert(a1 > ".")
+    assert(a1 == "bob@example.com")
+  end
+
+  def test_rmail_address_eql?
+    a1 = RMail::Address.new
+    a2 = RMail::Address.new
+
+    # Make sure it works on uninitialized addresses
+    assert(a1.eql?(a2))
+
+    # Display name and comments make no difference
+    a1.display_name = 'foo'
+    a1.comments = 'bar'
+    assert(a1.eql?(a2))
+
+    a1 = RMail::Address.new("Zeus <bob@example.com>")
+    assert_exception(TypeError) {
+      a1.eql?("bob@eaxample.com")
+    }
+  end
+
+  def test_rmail_address_hash
+    assert_no_exception {
+      RMail::Address.new.hash
+    }
+    a1 = RMail::Address.new("Zeus <bob@example.com>")
+    assert_equal(a1.hash, "bob@example.com".hash)
+  end
+
 end
